@@ -15,8 +15,6 @@ import re
 
 import os.path as op
 
-from functools import partial
-
 from datalad.config import ConfigManager
 from datalad.interface.base import Interface
 from datalad.interface.utils import eval_results
@@ -25,7 +23,6 @@ from datalad.interface.results import (
     get_status_dict,
     results_from_paths,
     annexjson2result,
-    count_results,
     success_status_map,
     results_from_annex_noinfo,
 )
@@ -61,9 +58,6 @@ from datalad.support.network import (
 from datalad.support.parallel import (
     ProducerConsumerProgressLog,
 )
-from datalad.dochelpers import (
-    single_or_plural,
-)
 from datalad.utils import (
     unique,
     Path,
@@ -112,20 +106,20 @@ def _get_flexible_source_candidates_for_submodule(ds, sm):
     cost is given in parenthesis, higher values indicate higher cost, and
     thus lower priority:
 
-    - URL of any configured superdataset remote that is known to have the
-      desired submodule commit, with the submodule path appended to it.
-      There can be more than one candidate (cost 500).
-
     - A datalad URL recorded in `.gitmodules` (cost 590). This allows for
       datalad URLs that require additional handling/resolution by datalad, like
       ria-schemes (ria+http, ria+ssh, etc.)
 
     - A URL or absolute path recorded for git in `.gitmodules` (cost 600).
 
+    - URL of any configured superdataset remote that is known to have the
+      desired submodule commit, with the submodule path appended to it.
+      There can be more than one candidate (cost 650).
+
     - In case `.gitmodules` contains a relative path instead of a URL,
       the URL of any configured superdataset remote that is known to have the
       desired submodule commit, with this relative path appended to it.
-      There can be more than one candidate (cost 500).
+      There can be more than one candidate (cost 650).
 
     - In case `.gitmodules` contains a relative path as a URL, the absolute
       path of the superdataset, appended with this relative path (cost 900).
@@ -219,7 +213,7 @@ def _get_flexible_source_candidates_for_submodule(ds, sm):
                 sm_path_url = sm_path
 
             clone_urls.extend(
-                dict(cost=500, name=remote, url=url)
+                dict(cost=650, name=remote, url=url)
                 for url in _get_flexible_source_candidates(
                     # alternate suffixes are tested by `clone` anyways
                     sm_path_url, remote_url, alternate_suffix=False)
@@ -448,7 +442,7 @@ def _install_necessary_subdatasets(
                                      on_failure='ignore',
                                      result_filter=is_ok_dataset)
         if not subds_trail:
-            # no (newly available) subdataset get's us any closer
+            # no (newly available) subdataset gets us any closer
             return
         # next round
         cur_subds = subds_trail[-1]
@@ -865,7 +859,8 @@ class Get(Interface):
                 recursive=True if path else recursive,
                 recursion_limit=None if path else recursion_limit,
                 return_type='generator',
-                on_failure='ignore'):
+                on_failure='ignore',
+                result_renderer='disabled'):
             if sdsres.get('type', None) != 'dataset':
                 # if it is not about a 'dataset' it is likely content in
                 # the root dataset
